<?php
/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 *
 * @author Sascha Nitsch (grumpydeveloper)
 **/

namespace Federator\DIO;

/**
 * IO functions related to users
 */
class Posts
{

    /**
     * get posts by user
     *
     * @param \mysqli $dbh @unused-param
     *          database handle
     * @param string $id
     *          user id
     * @param \Federator\Connector\Connector $connector
     *          connector to fetch use with
     * @param \Federator\Cache\Cache|null $cache
     *          optional caching service
     * @param string $minId
     *          minimum ID
     * @param string $maxId
     *          maximum ID
     * @return \Federator\Data\ActivityPub\Common\APObject[]
     */
    public static function getPostsByUser($dbh, $id, $connector, $cache, $minId, $maxId)
    {
        // ask cache
        if ($cache !== null) {
            $posts = $cache->getRemotePostsByUser($id, $minId, $maxId);
            if ($posts !== false) {
                return $posts;
            }
        }
        $posts = [];
        // TODO: check our db

        if ($posts === []) {
            // ask connector for user-id
            $posts = $connector->getRemotePostsByUser($id, $minId, $maxId);
            if ($posts === false) {
                $posts = [];
            }
        }
        // save posts to DB
        if ($cache !== null) {
            $cache->saveRemotePostsByUser($id, $posts);
        }
        return $posts;
    }
}
